=== modified file 'NEWS'
--- NEWS	2007-03-01 04:26:55 +0000
+++ NEWS	2007-03-01 07:15:55 +0000
@@ -151,6 +151,9 @@
     * ``bzr init`` works with unicode argument LOCATION.
       (Alexander Belchenko, #85599)
 
+    * Raise DependencyNotPresent if pycurl do not support https. (Vincent
+      Ladeuil, #85305)
+
   TESTING:
 
     * New ``--first`` option to ``bzr selftest`` to run specified tests

=== modified file 'bzr'
--- bzr	2007-02-20 07:16:27 +0000
+++ bzr	2007-02-28 18:39:16 +0000
@@ -104,7 +104,12 @@
     # are just about to be discarded anyhow.  This does mean that atexit hooks
     # won't run but we don't use them.  Also file buffers won't be flushed,
     # but our policy is to always close files from a finally block. -- mbp 20070215
-    sys.stdout.flush()
+    try:
+        sys.stdout.flush()
+    except IOError, e:
+        import errno
+        if sys.platform != 'win32' or e.errno != errno.EINVAL:
+            raise
     if bzrlib.trace._trace_file:
         # this is also _bzr_log
         bzrlib.trace._trace_file.flush()

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-03-01 06:47:29 +0000
+++ bzrlib/branch.py	2007-03-01 07:15:55 +0000
@@ -1451,7 +1451,9 @@
                 if not overwrite:
                     raise
             if overwrite:
-                self.set_revision_history(source.revision_history())
+                if stop_revision is None:
+                    stop_revision = source.last_revision()
+                self.generate_revision_history(stop_revision)
             result.tag_conflicts = source.tags.merge_to(self.tags)
             result.new_revno, result.new_revid = self.last_revision_info()
             if _hook_master:

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-12-19 17:37:38 +0000
+++ bzrlib/commands.py	2007-02-28 18:39:16 +0000
@@ -627,7 +627,7 @@
                 raise
             if e.errno != errno.EPIPE:
                 # Win32 raises IOError with errno=0 on a broken pipe
-                if sys.platform != 'win32' or e.errno != 0:
+                if sys.platform != 'win32' or (e.errno not in (0, errno.EINVAL)):
                     raise
             pass
         except KeyboardInterrupt:

=== modified file 'bzrlib/tests/branch_implementations/test_pull.py'
--- bzrlib/tests/branch_implementations/test_pull.py	2007-03-01 04:15:28 +0000
+++ bzrlib/tests/branch_implementations/test_pull.py	2007-03-01 07:15:55 +0000
@@ -80,6 +80,19 @@
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 checkout.branch.pull, other.branch)
 
+    def test_pull_overwrite(self):
+        tree_a = self.make_branch_and_tree('tree_a')
+        tree_a.commit('message 1')
+        tree_b = tree_a.bzrdir.sprout('tree_b').open_workingtree()
+        tree_a.commit('message 2', rev_id='rev2a')
+        tree_b.commit('message 2', rev_id='rev2b')
+        self.assertRaises(errors.DivergedBranches, tree_a.pull, tree_b.branch)
+        tree_a.branch.pull(tree_a.branch, overwrite=True,
+                           stop_revision='rev2b')
+        self.assertEqual('rev2b', tree_a.branch.last_revision())
+        self.assertEqual(tree_b.branch.revision_history(),
+                         tree_a.branch.revision_history())
+
 
 class TestPullHook(TestCaseWithBranch):
 

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2007-02-08 17:01:20 +0000
+++ bzrlib/tests/test_http.py	2007-02-28 21:40:44 +0000
@@ -126,10 +126,23 @@
         self.port = None
 
 
+class TestWithTransport_pycurl(object):
+    """Test case to inherit from if pycurl is present"""
+
+    def _get_pycurl_maybe(self):
+        try:
+            from bzrlib.transport.http._pycurl import PyCurlTransport
+            return PyCurlTransport
+        except errors.DependencyNotPresent:
+            raise TestSkipped('pycurl not present')
+
+    _transport = property(_get_pycurl_maybe)
+
+
 class TestHttpUrls(TestCase):
 
-    # FIXME: Some of these tests should be done for both
-    # implementations
+    # TODO: This should be moved to authorization tests once they
+    # are written.
 
     def test_url_parsing(self):
         f = FakeManager()
@@ -142,46 +155,102 @@
         self.assertEquals([None, 'www.bazaar-vcs.org', 'user', 'pass'],
                           f.credentials[0])
 
+
+class TestHttpTransportUrls(object):
+    """Test the http urls.
+
+    This MUST be used by daughter classes that also inherit from
+    TestCase.
+
+    We can't inherit directly from TestCase or the
+    test framework will try to create an instance which cannot
+    run, its implementation being incomplete.
+    """
+
     def test_abs_url(self):
         """Construction of absolute http URLs"""
-        t = HttpTransport_urllib('http://bazaar-vcs.org/bzr/bzr.dev/')
+        t = self._transport('http://bazaar-vcs.org/bzr/bzr.dev/')
         eq = self.assertEqualDiff
-        eq(t.abspath('.'),
-           'http://bazaar-vcs.org/bzr/bzr.dev')
-        eq(t.abspath('foo/bar'),
-           'http://bazaar-vcs.org/bzr/bzr.dev/foo/bar')
-        eq(t.abspath('.bzr'),
-           'http://bazaar-vcs.org/bzr/bzr.dev/.bzr')
+        eq(t.abspath('.'), 'http://bazaar-vcs.org/bzr/bzr.dev')
+        eq(t.abspath('foo/bar'), 'http://bazaar-vcs.org/bzr/bzr.dev/foo/bar')
+        eq(t.abspath('.bzr'), 'http://bazaar-vcs.org/bzr/bzr.dev/.bzr')
         eq(t.abspath('.bzr/1//2/./3'),
            'http://bazaar-vcs.org/bzr/bzr.dev/.bzr/1/2/3')
 
     def test_invalid_http_urls(self):
         """Trap invalid construction of urls"""
-        t = HttpTransport_urllib('http://bazaar-vcs.org/bzr/bzr.dev/')
-        self.assertRaises(ValueError,
-            t.abspath,
-            '.bzr/')
-        t = HttpTransport_urllib('http://http://bazaar-vcs.org/bzr/bzr.dev/')
-        self.assertRaises(errors.InvalidURL, t.has, 'foo/bar')
+        t = self._transport('http://bazaar-vcs.org/bzr/bzr.dev/')
+        self.assertRaises(ValueError, t.abspath, '.bzr/')
+        t = self._transport('http://http://bazaar-vcs.org/bzr/bzr.dev/')
+        self.assertRaises((errors.InvalidURL, errors.ConnectionError),
+                          t.has, 'foo/bar')
 
     def test_http_root_urls(self):
         """Construction of URLs from server root"""
-        t = HttpTransport_urllib('http://bzr.ozlabs.org/')
+        t = self._transport('http://bzr.ozlabs.org/')
         eq = self.assertEqualDiff
         eq(t.abspath('.bzr/tree-version'),
            'http://bzr.ozlabs.org/.bzr/tree-version')
 
     def test_http_impl_urls(self):
         """There are servers which ask for particular clients to connect"""
-        server = HttpServer_PyCurl()
+        server = self._server()
         try:
             server.setUp()
             url = server.get_url()
-            self.assertTrue(url.startswith('http+pycurl://'))
+            self.assertTrue(url.startswith('%s://' % self._qualified_prefix))
         finally:
             server.tearDown()
 
 
+class TestHttpUrls_urllib(TestHttpTransportUrls, TestCase):
+    """Test http urls with urllib"""
+
+    _transport = HttpTransport_urllib
+    _server = HttpServer_urllib
+    _qualified_prefix = 'http+urllib'
+
+
+class TestHttpUrls_pycurl(TestWithTransport_pycurl, TestHttpTransportUrls,
+                          TestCase):
+    """Test http urls with pycurl"""
+
+    _server = HttpServer_PyCurl
+    _qualified_prefix = 'http+pycurl'
+
+    # TODO: This should really be moved into another pycurl
+    # specific test. When https tests will be implemented, take
+    # this one into account.
+    def test_pycurl_without_https_support(self):
+        """Test that pycurl without SSL do not fail with a traceback.
+
+        For the purpose of the test, we force pycurl to ignore
+        https by supplying a fake version_info that do not
+        support it.
+        """
+        try:
+            import pycurl
+        except ImportError:
+            raise TestSkipped('pycurl not present')
+        # Now that we have pycurl imported, we can fake its version_info
+        # This was taken from a windows pycurl without SSL
+        # (thanks to bialix)
+        pycurl.version_info = lambda : (2,
+                                        '7.13.2',
+                                        462082,
+                                        'i386-pc-win32',
+                                        2576,
+                                        None,
+                                        0,
+                                        None,
+                                        ('ftp', 'gopher', 'telnet',
+                                         'dict', 'ldap', 'http', 'file'),
+                                        None,
+                                        0,
+                                        None)
+        self.assertRaises(errors.DependencyNotPresent, self._transport,
+                          'https://launchpad.net')
+
 class TestHttpConnections(object):
     """Test the http connections.
 
@@ -247,19 +316,6 @@
             socket.setdefaulttimeout(default_timeout)
 
 
-class TestWithTransport_pycurl(object):
-    """Test case to inherit from if pycurl is present"""
-
-    def _get_pycurl_maybe(self):
-        try:
-            from bzrlib.transport.http._pycurl import PyCurlTransport
-            return PyCurlTransport
-        except errors.DependencyNotPresent:
-            raise TestSkipped('pycurl not present')
-
-    _transport = property(_get_pycurl_maybe)
-
-
 class TestHttpConnections_urllib(TestHttpConnections, TestCaseWithWebserver):
     """Test http connections with urllib"""
 

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2007-02-26 10:02:58 +0000
+++ bzrlib/transport/http/_pycurl.py	2007-03-01 03:08:30 +0000
@@ -81,6 +81,12 @@
 
     def __init__(self, base, from_transport=None):
         super(PyCurlTransport, self).__init__(base)
+        if base.startswith('https'):
+            # Check availability of https into pycurl supported
+            # protocols
+            supported = pycurl.version_info()[8]
+            if 'https' not in supported:
+                raise DependencyNotPresent('pycurl', 'no https support')
         self.cabundle = ca_bundle.get_ca_path()
         if from_transport is not None:
             self._curl = from_transport._curl

